let selectedRows = new Set()

function saveSampleMetadata(rowData) {
	const fields = ['name', 'bpm', 'root', 'key', 'tags']
	const metadata = fields.reduce((obj, field, index) => {
		obj[field] = rowData[index]
		return obj
	}, {})

	metadata.action = 'saveSampleMetadata'
	metadata.packName = editingPack

	sendMessage(metadata)
}

function setPackSampleList(packsJson) {
	const json = JSON.parse(packsJson)
	if (json.packName !== editingPack) {
		console.error(`Pack name ${json.packName} doesnt match the expected ${editingPack}`)
		return
	}
	document.getElementById('pack-samples-list-title').textContent =
		'Pack Samples (' + json.samples.length + ')'

	const tableBody = document.getElementById('pack-samples-list')
	tableBody.innerHTML = ''

	json.samples.forEach((sampleName, index) => {
		const row = document.createElement('tr')
		row.dataset.index = index
		row.dataset.id = sampleName

		row.appendChild(makeLabelCell(69, true))
		row.appendChild(makeLabelCell(sampleName))
		row.appendChild(makeTextInputField('0.00'))
		row.appendChild(makeTextInputField(' '))
		row.appendChild(makeDropDownField('none', supportedKeys))
		row.appendChild(makeTextInputField(' ', false))

		tableBody.appendChild(row)

		sendMessage({
			action: 'requestSampleMetadata',
			packName: editingPack,
			name: sampleName,
		})
	})
}

function onRowClicked(row, event) {}

function setSampleMetadata(sampleMetadata) {
	const metadata = JSON.parse(sampleMetadata)
	if (metadata['packName'] !== editingPack) {
		return
	}

	const tableBody = document.getElementById('pack-samples-list')
	const rows = tableBody.querySelectorAll('tr')

	rows.forEach((row) => {
		if (row.dataset.id === metadata['name']) {
			const fields = row.querySelectorAll('input, select')
			const cells = row.querySelectorAll('td')
			cells[0].textContent = metadata['id']
			cells[1].textContent = metadata['name']
			fields[0].value = metadata['bpm'] ? metadata['bpm'].toString() : ''
			fields[1].value = metadata['root'] || ''
			fields[2].value = metadata['key'] || 'none'
			fields[3].value = metadata['tags'] || ''
		}
	})

	let lastSelectedRowIndex = null

	rows.forEach((row, index) => {
		row.addEventListener('click', (event) => {
			if (event.metaKey) {
				if (selectedRows.has(index)) {
					selectedRows.delete(index)
					rows[index].classList.remove('selected-tr')
				} else {
					selectedRows.add(index)
					rows[index].classList.add('selected-tr')
				}
				lastSelectedRowIndex = index
			} else if (event.shiftKey && lastSelectedRowIndex !== null) {
				const start = Math.min(lastSelectedRowIndex, index)
				const end = Math.max(lastSelectedRowIndex, index)

				selectedRows.forEach((i) => rows[i].classList.remove('selected-tr'))
				selectedRows.clear()

				for (let i = start; i <= end; i++) {
					selectedRows.add(i)
					rows[i].classList.add('selected-tr')
				}
			} else {
				selectedRows.forEach((i) => rows[i].classList.remove('selected-tr'))
				selectedRows.clear()
				selectedRows.add(index)
				row.classList.add('selected-tr')
				lastSelectedRowIndex = index
			}
			event.stopImmediatePropagation()
		})
	})
}

function getNamesOfSelectedCells() {
	const rows = Array.from(document.getElementById('pack-samples-list').querySelectorAll('tr'))

	const firstColumnValues = Array.from(selectedRows).map((index) => {
		const cell = rows[index].querySelector('td:nth-child(2)')
		return cell ? cell.textContent.trim() : null
	})
	return firstColumnValues
}

function setAllSampleMetadata(sampleMetadata) {
	const metadata = JSON.parse(sampleMetadata)
	if (metadata['packName'] !== editingPack) {
		return
	}

	const sampleLayout = organiseSamples(metadata.samples, getNamesOfSelectedCells())
	sendMessage({
		action: 'assignPads',
		packName: editingPack,
		samples: sampleLayout,
	})
}
