function makeDropDownField(value, options) {
	const cell = document.createElement('td')
	cell.setAttribute('data-value', 'text')
	cell.setAttribute('data-type', 'dropdown')

	const dropdown = document.createElement('select')
	options.forEach((option) => {
		const optionElement = document.createElement('option')
		optionElement.value = option
		optionElement.textContent = option
		dropdown.appendChild(optionElement)
	})
	dropdown.value = value
	dropdown.addEventListener('change', function () {
		const rowValues = Array.from(event.target.closest('tr').querySelectorAll('td')).map(
			(cell) => cell.textContent || cell.querySelector('input')?.value
		)
		saveSampleMetadata(rowValues)
	})

	cell.appendChild(dropdown)
	return cell
}

function makeTextInputField(value, centered = true) {
	const cell = document.createElement('td')
	const field = document.createElement('input')
	field.type = 'text'
	field.value = value
	field.className = centered ? 'centered-input' : ''

	field.addEventListener('blur', (event) => {
		const rowValues = Array.from(event.target.closest('tr').querySelectorAll('td')).map(
			(cell) => cell.textContent || cell.querySelector('input')?.value
		)
		saveSampleMetadata(rowValues)
	})

	field.addEventListener('keydown', (event) => {
		if (event.key === 'Enter') {
			event.target.blur()
		}
	})
	cell.appendChild(field)
	return cell
}

function makeLabelCell(value, centered = false) {
	const itemCell = document.createElement('td')
	itemCell.textContent = value
	if (centered) {
		itemCell.style.textAlign = 'center'
	}
	return itemCell
}

function makeBooleanCell(value) {
	const itemCell = document.createElement('td')
	itemCell.style.textAlign = 'center'
	itemCell.innerHTML =
		value === 'true'
			? '<span style="font-size: 1.25em; color: green; display: inline-block;">✓</span>'
			: '<span style="font-size: 0.75em; color: red; display: inline-block;">❌</span>'
	return itemCell
}

function makeButtonCell(buttonText, callback) {
	const itemCell = document.createElement('td')
	itemCell.style.textAlign = 'center'

	const button = document.createElement('button')
	button.textContent = buttonText
	button.addEventListener('click', callback)

	itemCell.appendChild(button)
	return itemCell
}

function limit(value, scalar = 100000) {
	return Math.round(value * scalar) / scalar
}
